<?php

/**
 * Plugin Name:     Wpct i18n
 * Plugin URI:      https://git.coopdevs.org/codeccoop/wp/wpct-i18n
 * Description:     String translation
 * Author:          Còdec Cooperativa
 * Author URI:      https://www.codeccoop.org
 * Text Domain:     wpct-i18n
 * Domain Path:     /languages
 * Version:         2.1.11
 *
 * @package         Wpct_i18n
 */

namespace WPCT_I18N;

use WPCT_ABSTRACT\Plugin as Base_Plugin;
use WP_Post;
use WP_Term;

if (!defined('ABSPATH')) {
    exit;
}

if (!class_exists('\WPCT_I18N\Wpct_i18n')) :

    if (is_file(plugin_dir_path(__FILE__) . 'abstracts/class-plugin.php')) {
        include_once(plugin_dir_path(__FILE__) . 'abstracts/class-plugin.php');
    }

    /**
     * I18N class.
     *
     * Adapter to interact with WPML and Polylang translation systems.
     */
    class Wpct_i18n extends Base_Plugin
    {
        /**
         * Handles current integration.
         *
         * @var string
         */
        private static $integration;

        /**
         * Plugin initializer
         */
        protected static function init()
        {
            // Configure integration
            if (self::is_plugin_active('polylang/polylang.php') && function_exists('pll_current_language')) {
                self::$integration = 'pll';
            } elseif (self::is_plugin_active('sitepress-multilingual-cms/sitepress.php')) {
                self::$integration = 'wpml';
            }

            // Register custom post status
            self::trans_status();

            // Hooks
            self::custom_hooks();
        }

        private static function secure_call($fn, ...$args)
        {
            if (function_exists($fn)) {
                return call_user_func($fn, ...$args);
            }
        }

        /**
         * Bind plugin custom hooks.
         */
        private static function custom_hooks()
        {
            add_filter('wpct_i18n_default_language', static function ($default, $format = 'slug') {
                return self::default_language($format);
            }, 10, 2);

            add_filter('wpct_i18n_current_language', static function ($default, $format = 'slug') {
                return self::current_language($format);
            }, 10, 2);

            add_filter('wpct_i18n_active_languages', static function ($default, $format = 'slug') {
                return self::active_languages($format);
            }, 10, 2);

            add_filter('wpct_i18n_post_language', static function ($default, $post_id, $format = 'slug') {
                return self::post_language($post_id, $format);
            }, 10, 3);

            add_filter('wpct_i18n_post_translations', static function ($default, $post_id) {
                return self::post_translations($post_id);
            }, 10, 2);

            add_filter('wpct_i18n_term_translations', static function ($default, $term_id, $tax_name) {
                return self::term_translations($term_id, $tax_name);
            }, 10, 3);

            add_filter('wpct_i18n_is_translation', static function ($default, $post_id) {
                return self::is_post_translation($post_id);
            }, 10, 2);

            add_filter('wpct_i18n_is_term_translation', static function ($default, $term_id, $tax_name) {
                return self::is_term_translation($term_id, $tax_name);
            }, 10, 3);

            add_action('wpct_i18n_translate_post', static function ($post, $language, &$trans_id = null) {
                $trans_id = self::translate_post($post, $language);
            }, 10, 3);

            // add_action('wpct_i18n_translate_term', static function ($term, $language, &$trans_id) {
            //     $trans_id = self::translate_term($term, $language);
            // }, 10, 3);

            add_action('wpct_i18n_set_post_language', static function ($post, $language) {
                self::set_post_language($post, $language);
            }, 10, 2);

            add_action('wpct_i18n_set_term_language', static function ($term, $tax_name, $language) {
                self::set_term_language($term, $tax_name, $language);
            }, 10, 3);

            add_filter('wpct_i18n_language_switcher', static function ($default, $echo = false) {
                return self::language_switcher($echo);
            }, 10, 2);

            add_shortcode('wpct_i18n_language_switcher', static function () {
                return self::language_switcher(false);
            });
        }

        /**
        * Transform locale strings into 2 char slug.
        *
        * @param string $locale IETF's BPC 47 locale format.
        *
        * @return string Language slug.
        */
        private static function locale_to_slug($locale)
        {
            return strtolower(substr($locale, 0, 2));
        }

        /**
         * Gets WPML language object by slug.
         *
         * @param string $slug Language slug.
         *
         * @return object|null $language Language instance.
         */
        private static function get_wpml_language_by_slug($slug)
        {
            $languages = apply_filters('wpml_active_languages', null);

            $codes = array_values(array_filter(array_keys($languages), static function ($code) use ($slug, $languages) {
                $lang = $languages[$code];
                return $lang['language_code'] === $slug;
            }));

            if (count($codes) === 0) {
                return null;
            }

            return $languages[$codes[0]] ?? null;
        }

        /**
         * Returns the site's default language.
         *
         * @param string $format Return format (slug or locale).
         *
         * @return string|null Site's default language.
         */
        private static function default_language($format = 'slug')
        {
            switch (self::$integration) {
                case 'pll':
                    return self::secure_call('pll_default_language', $format);

                case 'wpml':
                    $slug = apply_filters('wpml_default_language', null);

                    if ($format === 'slug') {
                        return $slug;
                    } else {
                        $language = self::get_wpml_language_by_slug($slug);
                        return $language['default_locale'] ?? null;
                    }

                    // no break
                default:
                    $locale = get_locale();

                    if ($format === 'locale') {
                        return $locale;
                    }

                    return self::locale_to_slug($locale);
            }
        }

        /**
         * Returns the site's current language.
         *
         * @param string $format Return format (slug or locale).
         *
         * @return string Site's current language.
         */
        private static function current_language($format = 'slug')
        {
            switch (self::$integration) {
                case 'pll':
                    return self::secure_call('pll_current_language', $format);

                case 'wpml':
                    $slug = apply_filters('wpml_current_language', null);

                    if ($format === 'slug') {
                        return $slug;
                    } else {
                        $language = self::get_wpml_language_by_slug($slug);
                        return $language['default_locale'];
                    }

                    // no break
                default:
                    $locale = get_locale();

                    if ($format === 'locale') {
                        return $locale;
                    }

                    return self::locale_to_slug($locale);
            }
        }

        /**
         * Returns the site's active languages.
         *
         * @param string $format Return format (slug or locale).
         * @return string Site's active languages.
         */
        private static function active_languages($format = 'slug')
        {
            switch (self::$integration) {
                case 'pll':
                    $languages = self::secure_call('pll_languages_list', ['fields' => $format, 'hide_empty' => false]);

                    return (array) $languages;

                case 'wpml':
                    $languages = apply_filters('wpml_active_languages', null);
                    $field = $format === 'slug' ? 'language_code' : 'default_locale';

                    return array_map(function ($lng) use ($field) {
                        return $lng[$field];
                    }, $languages);

                default:
                    $languages = [get_locale()];

                    if ($format === 'locale') {
                        return $languages;
                    }

                    return array_map(function ($locale) {
                        return self::locale_to_slug($locale);
                    }, $languages);
            }
        }

        /**
         * Renders the pair plugin default language switcher.
         *
         * @param boolean $echo Control if it should echo the switcher.
         *
         * @return string|null Language switcher rendered HTML.
         */
        private static function language_switcher($echo)
        {
            if (!$echo) {
                ob_start();
            }

            switch (self::$integration) {
                case 'pll':
                    self::secure_call('pll_the_languages');
                    break;
                case 'wpml':
                    do_action('wpml_add_language_selector');
                    break;
                default:
                    echo '';
            }

            if (!$echo) {
                return ob_get_clean();
            }
        }

        /**
         * Gets WP_Post language.
         *
         * @param WP_Post|int $post_id Post ID.
         * @param string $format Return format (slug or locale).
         *
         * @return string|null Post's language.
         */
        private static function post_language($post_id, $format = 'slug')
        {
            if ($post_id instanceof WP_Post) {
                $post_id = $post_id->ID;
            }

            switch (self::$integration) {
                case 'pll':
                    return self::secure_call('pll_get_post_language', $post_id, $format);

                case 'wpml':
                    $details = apply_filters('wpml_post_language_details', null, $post_id);

                    if (empty($details)) {
                        return;
                    }

                    if ($format === 'slug') {
                        return $details['language_code'];
                    } else {
                        return $details['locale'];
                    }

                    // no break
                default:
                    $locale = get_locale();

                    if ($format === 'locale') {
                        return $locale;
                    } else {
                        return self::locale_to_slug($locale);
                    }

            }
        }

        /**
         * Gets WP_Post current translations.
         *
         * @param WP_Post|int $post_id Post ID.
         *
         * @return array<integer, string> Array with the post's translations IDs.
         */
        private static function post_translations($post_id)
        {
            if ($post_id instanceof WP_Post) {
                $post_id = $post_id->ID;
            }

            if (empty($post_id)) {
                return [];
            }

            switch (self::$integration) {
                case 'pll':
                    return self::secure_call('pll_get_post_translations', $post_id);

                case 'wpml':
                    $type = 'post_' . get_post_type($post_id);
                    $trid = apply_filters('wpml_element_trid', false, $post_id, $type);

                    $translations = [];
                    $post_translations = apply_filters('wpml_get_element_translations', [], $trid, $type);
                    foreach ($post_translations as $trans) {
                        $translations[$trans->language_code] = (int) $trans->element_id;
                    }

                    return $translations;

                default:
                    $current_lang = self::current_language();
                    return [$current_lang => $post_id];
            }
        }

        /**
         * Gets WP_Term current translations.
         *
         * @param WP_Term|int $term_id Term ID.
         * @param string $tax_name Name of the term's taxonomy.
         *
         * @return array<integer, string> Array with the term's translations IDs.
         */
        private static function term_translations($term_id, $tax_name)
        {
            if ($term_id instanceof WP_Term) {
                $term_id = $term_id->term_id;
            }

            if (empty($term_id)) {
                return [];
            }

            switch (self::$integration) {
                case 'pll':
                    return self::secure_call('pll_get_term_translations', $term_id);

                case 'wpml':
                    $type = 'tax_' . $tax_name;
                    $trid = apply_filters('wpml_element_trid', false, $term_id, $type);

                    $translations = [];
                    $term_translations = apply_filters('wpml_get_element_translations', [], $trid, $type);
                    foreach ($term_translations as $trans) {
                        $translations[$trans->language_code] = (int) $trans->element_id;
                    }

                    return $translations;

                default:
                    $current_lang = self::current_language();
                    return [$current_lang => $term_id];
            }
        }

        /**
         * Checks if the post is a translation.
         *
         * @param WP_Post|int $post_id Post ID.
         *
         * @return boolean True if post is a translation.
         */
        private static function is_post_translation($post_id)
        {
            if ($post_id instanceof WP_Post) {
                $post_id = $post_id->ID;
            }

            if (empty($post_id)) {
                return;
            }

            $default = self::default_language();

            switch (self::$integration) {
                case 'pll':
                    $post_lang = self::secure_call('pll_get_post_language', $post_id);
                    return !empty($post_lang) ? $post_lang !== $default : false;

                case 'wpml':
                    $lang_details = apply_filters('wpml_post_language_details', null, $post_id) ?: ['language_code' => $default];
                    return $lang_details['language_code'] !== self::default_language();

                default:
                    return false;
            }
        }

        /**
         * Checks if the term is a translation.
         *
         * @param WP_Term|int $term_id Term ID.
         * @param string $tax_name Name of the term's taxonomy.
         *
         * @return boolean True if term is a translation.
         */
        private static function is_term_translation($term_id, $tax_name)
        {
            if ($term_id instanceof WP_Term) {
                $term_id = $term_id->term_id;
            }

            if (empty($term_id)) {
                return;
            }

            $default = self::default_language();

            switch (self::$integration) {
                case 'pll':
                    $term_lang = self::Secure_call('pll_get_term_language', $term_id);
                    return !empty($term_lang) ? $term_lang !== $default : false;

                case 'wpml':
                    $lang_details = ((array) apply_filters(
                        'wpml_element_language_details',
                        null,
                        [
                            'element_id' => $term_id,
                            'element_type' => $tax_name,
                        ]
                    ) ?: ['language_code' => $default]);

                    return $lang_details['language_code'] !== self::default_language();

                default:
                    return false;
            }
        }

        /**
         * Creates post translation.
         *
         * @param WP_Post|int $post WP_Post instance.
         * @param string $lang Language code.
         *
         * @return int|null Translation ID.
         */
        private static function translate_post($post, $lang)
        {
            if (empty(self::$integration)) {
                return;
            }

            if (is_int($post)) {
                $post = get_post($post);
            }

            $post_data = wp_slash((array) $post);
            $post_data['_thumbnail_id'] = get_post_thumbnail_id($post_data['ID']);
            unset($post_data['ID']);
            $post_data['post_status'] = 'translating';
            $trans_id = wp_insert_post($post_data);

            if (is_wp_error($trans_id)) {
                return;
            }

            switch (self::$integration) {
                case 'pll':
                    self::set_post_language($trans_id, $lang);

                    $translations = self::post_translations($post->ID);
                    $translations[$lang] = $trans_id;
                    self::secure_call('pll_save_post_translations', $translations);
                    break;
                case 'wpml':
                    self::set_post_language($trans_id, $lang);
                    break;
                default:
                    return;
            }

            wp_update_post([
                'ID' => $trans_id,
                'post_status' => $post->post_status
            ]);

            return $trans_id;
        }

        /**
         * Sets the post language.
         *
         * @param WP_Post|int $post_id Post ID.
         * @param string $lang Language code.
         */
        private static function set_post_language($post_id, $lang)
        {
            if (empty(self::$integration)) {
                return;
            }

            if ($post_id instanceof WP_Post) {
                $post_id = $post_id->ID;
            }

            if (empty($post_id)) {
                return;
            }

            switch (self::$integration) {
                case 'pll':
                    self::secure_call('pll_set_post_language', $post_id, $lang);
                    break;

                case 'wpml':
                    $is_translation = self::is_post_translation($post_id);

                    if ($is_translation) {
                        $translations = self::post_translations($post_id);
                        $default_language = self::default_language('slug');
                        $source_id = $translations[$default_language];
                    } else {
                        $source_id = $post_id;
                    }

                    $lang_details = apply_filters(
                        'wpml_post_language_details',
                        null,
                        $source_id,
                    ) ?: [
                        'trid' => false,
                        'language_code' => null,
                    ];

                    do_action('wpml_set_element_language_details', [
                        'element_id' => $post_id,
                        'element_type' => 'post_' . get_post_type($post_id),
                        'trid' => $lang_details['trid'],
                        'language_code' => $lang,
                        'source_language_code' => $lang_details['language_code'],
                    ]);
                    break;
            }
        }

        /**
         * Sets the term language.
         *
         * @param WP_Term|int $term_id Term ID.
         * @param string $tax_name Name of the term's taxonomy.
         * @param string $lang Language code.
         */
        private static function set_term_language($term_id, $tax_name, $lang)
        {
            if (empty(self::$integration)) {
                return;
            }

            if ($term_id instanceof WP_Term) {
                $term_id = $term_id->term_id;
            }

            if (empty($term_id)) {
                return;
            }

            switch (self::$integration) {
                case 'pll':
                    self::secure_call('pll_set_term_language', $term_id, $lang);
                    return $term_id;

                case 'wpml':
                    $is_translation = self::is_term_translation($term_id, $tax_name);

                    if ($is_translation) {
                        $translations = self::term_translations($term_id, $tax_name);
                        $default_language = self::default_language('slug');
                        $source_id = $translations[$default_language];
                    } else {
                        $source_id = $term_id;
                    }

                    $lang_details = ((array) apply_filters(
                        'wpml_element_language_details',
                        null,
                        $source_id
                    )) ?: [
                         'trid' => false,
                         'language_code' => null
                    ];

                    do_action('wpml_set_element_language_details', [
                        'element_id' => $term_id,
                        'element_type' => 'tax_' . $tax_name,
                        'trid' => $lang_details['trid'],
                        'language_code' => $lang,
                        'source_language_code' => $lang_details['language_code'],
                    ]);
            }
        }

        /**
         * Registers translation custom post status.
         */
        private static function trans_status()
        {
            register_post_status('translating', [
                'label' => __('Provisional status while translating post', 'wpct-i18n'),
                'public' => false,
                'internal' => true
            ]);
        }
    }

endif;

Wpct_i18n::setup();
